/*
 * Copyright 2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.verilog.parser.elaborator;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.util.InvariantChecks;

/**
 * {@link VerilogBinding} stores information about a binding.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogBinding {
  public enum Type {
    /** Right-hand-side expression is final. */
    FINAL,
    /** Current bindings should be applied. */
    CURRENT_CONTEXT,
    /** Parent bindings should be applied. */
    PARENT_CONTEXT
  }

  private final Type type;
  private final VerilogVariable lhs;
  private final Node rhs;

  public VerilogBinding(final Type type, final VerilogVariable lhs, final Node rhs) {
    InvariantChecks.checkNotNull(type);
    InvariantChecks.checkNotNull(lhs);
    InvariantChecks.checkNotNull(rhs);

    this.type = type;
    this.lhs = lhs;
    this.rhs = rhs;
  }

  public Type getType() {
    return type;
  }

  public VerilogVariable getLhs() {
    return lhs;
  }

  public Node getRhs() {
    return rhs;
  }

  @Override
  public String toString() {
    return rhs.toString();
  }
}
