/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.elaborator;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.VerilogDeclaration;
import ru.ispras.verilog.parser.model.basis.VerilogRange;
import ru.ispras.verilog.parser.model.util.ModelUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * {@link VerilogDescriptor} stores information about a design variable.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogDescriptor {
  private final VerilogDeclaration declaration;
  private final VerilogVariable variable;
  private final DataType dataType;
  private final int offset;
  private final List<VerilogIndexRange> indexRanges = new ArrayList<>();

  public VerilogDescriptor(final VerilogVariable variable, final VerilogDeclaration declaration) {
    InvariantChecks.checkNotNull(variable);
    InvariantChecks.checkNotNull(declaration);

    this.declaration = declaration;
    this.variable = variable;
    this.dataType = ModelUtils.getDataType(declaration, indexRanges);

    final VerilogRange range = declaration.getElementType().getRange();
    this.offset = range != null ? ModelUtils.getLowerBit(range) : 0;
  }

  public VerilogDeclaration getDeclaration() {
    return declaration;
  }

  public VerilogVariable getVariable() {
    return variable;
  }

  public DataType getDataType() {
    return dataType;
  }

  public int getOffset() {
    return offset;
  }

  public List<VerilogIndexRange> getIndexRanges() {
    return indexRanges;
  }

  @Override
  public String toString() {
    return String.format("%s:(SHIFT %d)", dataType, offset);
  }
}
