/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.basis.VerilogExpression;

import java.util.EnumSet;

/**
 * {@link VerilogPortConnection} represents port connections.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogPortConnection extends VerilogNode {
  public static final Tag TAG = Tag.PORT_CONNECTION;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.noneOf(Tag.class);

  /** Expression connected to the port. */
  private VerilogExpression expression;

  /**
   * Creates a port connection.
   * 
   * @param parent the parent node.
   */
  public VerilogPortConnection(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.ELEMENT, parent);
  }

  /**
   * Creates a copy of the port connection.
   * 
   * @param other the port connection to be copied.
   * @param parent the parent node.
   */
  public VerilogPortConnection(final VerilogPortConnection other, final VerilogNode parent) {
    super(other, parent);

    this.expression = other.expression != null ? other.expression.clone() : null;
  }

  /**
   * Returns the expression connected to the port.
   * 
   * @return the port connection expression.
   */
  public VerilogExpression getExpression() {
    return expression;
  }

  /**
   * Sets the expression connected to the port.
   * 
   * @param expression the expression to be set.
   */
  public void setExpression(final VerilogExpression expression) {
    InvariantChecks.checkNotNull(expression);
    this.expression = expression;
  }

  @Override
  public VerilogPortConnection clone() {
    return new VerilogPortConnection(this, getParentNode());
  }
}
