/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model;

import java.util.EnumSet;

/**
 * {@link VerilogSpecify} represents the abstract syntax of the specify construct.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogSpecify extends VerilogNode {
  public static final Tag TAG = Tag.SPECIFY;
  public static final EnumSet<Tag> TAGS_CHILDREN = EnumSet.of(
      Tag.DECLARATION,
      Tag.PULSE_STYLE,
      Tag.SHOW_CANCELLED,
      Tag.PATH_DECLARATION,
      Tag.TASK_STATEMENT
  );

  /**
   * Creates a specify construct.
   * 
   * @param parent the parent node.
   */
  public VerilogSpecify(final VerilogNode parent) {
    super(TAG, TAGS_CHILDREN, NodeKind.CLASS, parent);
  }

  /**
   * Creates a copy of the specify construct.
   * 
   * @param other the specify construct to be copied.
   * @param parent the parent node.
   */
  public VerilogSpecify(final VerilogSpecify other, final VerilogNode parent) {
    super(other, parent);
  }

  @Override
  public VerilogSpecify clone() {
    return new VerilogSpecify(this, getParentNode());
  }
}
