/*
 * Copyright 2012-2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.model.basis;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * {@link VerilogEvent} represents events.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogEvent {
  /** Event edge. */
  private VerilogEdge edge;
  /** Event expression. */
  private VerilogExpression expression;

  /**
   * Creates an event.
   */
  public VerilogEvent() {}

  /**
   * Creates a copy of the event.
   * 
   * @param other the event to be copied.
   */
  public VerilogEvent(final VerilogEvent other) {
    InvariantChecks.checkNotNull(other);

    this.edge = other.edge;
    this.expression = other.expression.clone();
  }

  /**
   * Returns the edge of the event.
   * 
   * @return the event edge.
   */
  public VerilogEdge getEdge() {
    return edge;
  }

  /**
   * Sets the edge of the event.
   * 
   * @param edge the edge to set.
   */
  public void setEdge(final VerilogEdge edge) {
    InvariantChecks.checkNotNull(edge);
    this.edge = edge;
  }

  /**
   * Returns the expression of the event.
   * 
   * @return the event expression.
   */
  public VerilogExpression getExpression() {
    return expression;
  }

  /**
   * Sets the expression of the event.
   * 
   * @param expression the expression to be set.
   */
  public void setExpression(final VerilogExpression expression) {
    InvariantChecks.checkNotNull(expression);
    this.expression = expression;
  }

  @Override
  public String toString() {
    return String.format("%s(%s)", edge, expression);
  }

  @Override
  public VerilogEvent clone() {
    return new VerilogEvent(this);
  }
}
