/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser.transformer;

import ru.ispras.fortress.expression.ExprUtils;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.transformer.Transformer;
import ru.ispras.fortress.transformer.TransformerRule;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.verilog.parser.model.VerilogDeclaration;
import ru.ispras.verilog.parser.model.VerilogNode;
import ru.ispras.verilog.parser.model.basis.VerilogReference;
import ru.ispras.verilog.parser.model.util.ModelUtils;

/**
 * {@link VerilogTransformerVariableRename} implements an engine that renames local variables.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class VerilogTransformerVariableRename extends VerilogTransformer {

  public static void transform(final VerilogNode node) {
    InvariantChecks.checkNotNull(node);

    final VerilogTransformer transformer = new VerilogTransformerVariableRename();
    transformer.run(node);
  }

  @Override
  public Node transform(final Node node) {
    InvariantChecks.checkNotNull(node);

    return Transformer.transform(node, Node.Kind.VARIABLE, new TransformerRule() {
      @Override
      public boolean isApplicable(final Node node) {
        return ExprUtils.isVariable(node);
      }

      @Override
      public Node apply(final Node node) {
        final NodeVariable oldVariable = (NodeVariable) node;
        final VerilogReference oldReference = ModelUtils.getReference(node);
        final VerilogDeclaration newDeclaration = oldReference.getDeclaration();

        final String oldName = oldVariable.getName();
        final String newName = newDeclaration.getName();

        if (oldName.equals(newName)) {
          return node;
        }

        final NodeVariable newVariable = new NodeVariable(newName, oldVariable.getDataType());
        newVariable.setUserData(oldReference);

        return newVariable;
      }
    });
  }
}
