/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.verilog.parser;

import org.apache.commons.io.FilenameUtils;
import ru.ispras.verilog.parser.sample.VerilogPrinter;

import java.nio.file.Paths;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * Basic class for test cases are aimed at collections of Verilog modules, i.e. benchmarks.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
abstract class VerilogBenchmarkTest {

  /**
   * Returns the full path to test directory.
   * @return The full path to test directory.
   */
  abstract String getTestDirPath();

  /**
   * Runs the tool on module with the specified name that is implemented in the array of files.
   * @param module The module name.
   * @param files The array of source files.
   */
  void runTest(final String module, final String... files) {

    final String moduleDir =
        Paths.get(
            getTestDirPath(),
            FilenameUtils.getFullPathNoEndSeparator(files[0])).toString();

    final String[] options = module == null
        ? new String[] { "--include-path", moduleDir }
        : new String[] { "--include-path", moduleDir, "--module-name", module };
    final String[] paths = new String[files.length];

    for (int i = 0; i < files.length; i++) {
      paths[i] = Paths.get(files[i]).toString();
    }

    runTest(options, paths);
  }

  private void runTest(final String[] options, final String[] paths) {
    final String[] fullPaths = new String[paths.length];

    for (int i = 0; i < paths.length; i++) {
      fullPaths[i] = getTestDirPath() + paths[i];
    }

    final List<String> commandLine = new LinkedList<>();
    commandLine.addAll(Arrays.asList(options));
    commandLine.addAll(Arrays.asList(fullPaths));

    VerilogPrinter.main(commandLine.toArray(new String[commandLine.size()]));
  }
}
