`timescale 1ns/1ps

// RAM consisting of four 32-bit items
module ram(clk, rst, val_rd, val_wr, addr_in, data_in, val_out, data_out, is_ready);
    input         clk;
    input         rst;
    input         val_rd;
    input         val_wr;
    input   [1:0] addr_in;
    input  [31:0] data_in;
    output        val_out;
    output [31:0] data_out;
    output        is_ready;

    reg    [31:0] mem0;
    reg    [31:0] mem1;
    reg    [31:0] mem2;
    reg    [31:0] mem3;

    reg    [31:0] result;
    reg     [1:0] state;

    // State encoding
    parameter RAM_IDLE   = 2'b00;
    parameter RAM_READ   = 2'b01;
    parameter RAM_WRITE  = 2'b10;
    parameter RAM_RESULT = 2'b11;

    assign is_ready = (state == RAM_IDLE);
    assign val_out  = (state == RAM_RESULT);
    assign data_out = (state == RAM_RESULT) ? result : 32'h0000_0000;

    always @(posedge rst)
        begin
            mem0   <= 32'hXXXX_XXXX;
            mem1   <= 32'hXXXX_XXXX;
            mem2   <= 32'hXXXX_XXXX;
            mem3   <= 32'hXXXX_XXXX;
            result <= 32'hXXXX_XXXX;
            state  <= 2'b00;
        end

    always @(posedge clk)
        begin
            if(state == RAM_IDLE)
              begin 
                // Read operation
                if(~rst & val_rd)
                  begin
                    state <= RAM_READ;

                    // Multiplexor 4-to-1
                    case(addr_in)
                    2'b00: result <= mem0;
                    2'b01: result <= mem1;
                    2'b10: result <= mem2;
                    2'b11: result <= mem3;
                    endcase
                  end
                // Write operation
                else if(~rst & val_wr)
                  begin
                    state  <= RAM_WRITE;
                    result <= 32'hXXXX_XXXX;

                    // Demultiplexor 1-to-4
                    case(addr_in)
                    2'b00: mem0 <= data_in;
                    2'b01: mem1 <= data_in;
                    2'b10: mem2 <= data_in;
                    2'b11: mem3 <= data_in;
                    endcase
                  end
              end
            else
              begin
                if(state == RAM_RESULT)
                  begin
                    state <= RAM_IDLE;
                  end
                else
                  begin
                    state <= RAM_RESULT;
                  end
              end
        end

endmodule /* ram */


